# This file was auto-generated by Fern from our API Definition.

import typing
from json.decoder import JSONDecodeError

from ...core.api_error import ApiError
from ...core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ...core.http_response import AsyncHttpResponse, HttpResponse
from ...core.pydantic_utilities import parse_obj_as
from ...core.request_options import RequestOptions
from ...core.serialization import convert_and_respect_annotation_metadata
from ...errors.unprocessable_entity_error import UnprocessableEntityError
from ...types.http_validation_error import HttpValidationError
from ...types.task_message import TaskMessage
from ...types.task_message_content import TaskMessageContent

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawBatchClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def create(
        self,
        *,
        contents: typing.Sequence[TaskMessageContent],
        task_id: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[typing.List[TaskMessage]]:
        """
        Parameters
        ----------
        contents : typing.Sequence[TaskMessageContent]

        task_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[typing.List[TaskMessage]]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "messages/batch",
            method="POST",
            json={
                "contents": convert_and_respect_annotation_metadata(
                    object_=contents, annotation=typing.Sequence[TaskMessageContent], direction="write"
                ),
                "task_id": task_id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.List[TaskMessage],
                    parse_obj_as(
                        type_=typing.List[TaskMessage],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def update(
        self,
        *,
        task_id: str,
        updates: typing.Dict[str, TaskMessageContent],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[typing.List[TaskMessage]]:
        """
        Parameters
        ----------
        task_id : str

        updates : typing.Dict[str, TaskMessageContent]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[typing.List[TaskMessage]]
            Successful Response
        """
        _response = self._client_wrapper.httpx_client.request(
            "messages/batch",
            method="PUT",
            json={
                "task_id": task_id,
                "updates": convert_and_respect_annotation_metadata(
                    object_=updates, annotation=typing.Dict[str, TaskMessageContent], direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.List[TaskMessage],
                    parse_obj_as(
                        type_=typing.List[TaskMessage],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)


class AsyncRawBatchClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def create(
        self,
        *,
        contents: typing.Sequence[TaskMessageContent],
        task_id: str,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[typing.List[TaskMessage]]:
        """
        Parameters
        ----------
        contents : typing.Sequence[TaskMessageContent]

        task_id : str

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[typing.List[TaskMessage]]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "messages/batch",
            method="POST",
            json={
                "contents": convert_and_respect_annotation_metadata(
                    object_=contents, annotation=typing.Sequence[TaskMessageContent], direction="write"
                ),
                "task_id": task_id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.List[TaskMessage],
                    parse_obj_as(
                        type_=typing.List[TaskMessage],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def update(
        self,
        *,
        task_id: str,
        updates: typing.Dict[str, TaskMessageContent],
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[typing.List[TaskMessage]]:
        """
        Parameters
        ----------
        task_id : str

        updates : typing.Dict[str, TaskMessageContent]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[typing.List[TaskMessage]]
            Successful Response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "messages/batch",
            method="PUT",
            json={
                "task_id": task_id,
                "updates": convert_and_respect_annotation_metadata(
                    object_=updates, annotation=typing.Dict[str, TaskMessageContent], direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    typing.List[TaskMessage],
                    parse_obj_as(
                        type_=typing.List[TaskMessage],  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 422:
                raise UnprocessableEntityError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        HttpValidationError,
                        parse_obj_as(
                            type_=HttpValidationError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)
